#  Draco Econometrics
#  Copyright (C) 2006-2009 Approximatrix, LLC
#  http://www.approximatrix.com
#  
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#  
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.


# The necessary java version
!define JRE_VERSION "1.6.0"

Function CheckInstalledJRE

  # MessageBox MB_OK "Checking Installed JRE Version"
  
  Push "${JRE_VERSION}"
  Call DetectJRE
  Exch $0   ; Get return value from stack
  StrCmp $0 "0" NoFound
  StrCmp $0 "-1" FoundOld
  
  Goto JREAlreadyInstalled
  
  FoundOld:
    #MessageBox MB_OK "Old JRE found"
    !insertmacro MUI_INSTALLOPTIONS_WRITE "jre.ini" "Field 1" "Text" "$(^Name) requires a more recent version of the Java Runtime Environment than the one found on your computer.  Please download version ${JRE_VERSION} or higher from:"
    !insertmacro MUI_INSTALLOPTIONS_WRITE "jre.ini" "Field 2" "Text" "http://java.com/"
    
    # Hides the next Button
    GetDlgItem $0 $HWNDPARENT 1
    ShowWindow $0 0
    
    Goto JRE_done
    
  NoFound:
    #MessageBox MB_OK "JRE not found"
    !insertmacro MUI_INSTALLOPTIONS_WRITE "jre.ini" "Field 1" "Text" "$(^Name) requires the Java Runtime Environment to be installed on your computer.  Please download Java from:"
    !insertmacro MUI_INSTALLOPTIONS_WRITE "jre.ini" "Field 2" "Text" "http://java.com/"
    
    # Hides the next Button
    GetDlgItem $0 $HWNDPARENT 1
    ShowWindow $0 0
    
    Goto JRE_done

  JREAlreadyInstalled:
    # No need to tell the user anything!  Java seems fine...
    Return
    
    #MessageBox MB_OK "JRE already installed"
    #!insertmacro MUI_INSTALLOPTIONS_WRITE "jre.ini" "Field 1" "Text" "The current version of Java is acceptable for use with $(^Name)"
    #!insertmacro MUI_INSTALLOPTIONS_WRITE "jre.ini" "Field 2" "Text" ""
    #Pop $0        ; Restore $0
    #Goto JRE_done

  JRE_done:
    !insertmacro MUI_HEADER_TEXT "Java Runtime Environment Detection" "To determine if the proper version of Java is installed"
    !insertmacro MUI_INSTALLOPTIONS_DISPLAY "jre.ini"
    
FunctionEnd

; Returns: 0 - JRE not found. -1 - JRE found but too old. Otherwise - Path to JAVA EXE
 
; DetectJRE. Version requested is on the stack.
; Returns (on stack)    "0" on failure (java too old or not installed), otherwise path to java interpreter
; Stack value will be overwritten!
 
Function DetectJRE
  Exch $0   ; Get version requested  
        ; Now the previous value of $0 is on the stack, and the asked for version of JDK is in $0
  Push $1   ; $1 = Java version string (ie 1.5.0)
  Push $2   ; $2 = Javahome
  Push $3   ; $3 and $4 are used for checking the major/minor version of java
  Push $4
  #MessageBox MB_OK "Detecting JRE"
  ReadRegStr $1 HKLM "SOFTWARE\JavaSoft\Java Runtime Environment" "CurrentVersion"
  #MessageBox MB_OK "Read : $1"
  StrCmp $1 "" DetectTry2
  ReadRegStr $2 HKLM "SOFTWARE\JavaSoft\Java Runtime Environment\$1" "JavaHome"
  #MessageBox MB_OK "Read 3: $2"
  StrCmp $2 "" DetectTry2
  Goto GetJRE
 
DetectTry2:
  ReadRegStr $1 HKLM "SOFTWARE\JavaSoft\Java Development Kit" "CurrentVersion"
  #MessageBox MB_OK "Detect Read : $1"
  StrCmp $1 "" NoFound
  ReadRegStr $2 HKLM "SOFTWARE\JavaSoft\Java Runtime Environment\$1" "JavaHome"
  #MessageBox MB_OK "Detect Read 3: $2"
  StrCmp $2 "" NoFound
 
GetJRE:
  ; $0 = version requested. $1 = version found. $2 = javaHome
  #MessageBox MB_OK "Getting JRE"
  IfFileExists "$2\bin\java.exe" 0 NoFound
  StrCpy $3 $0 1            ; Get major version. Example: $1 = 1.5.0, now $3 = 1
  StrCpy $4 $1 1            ; $3 = major version requested, $4 = major version found
  #MessageBox MB_OK "Want $3 , found $4"
  IntCmp $4 $3 0 FoundOld FoundNew
  StrCpy $3 $0 1 2
  StrCpy $4 $1 1 2          ; Same as above. $3 is minor version requested, $4 is minor version installed
  #MessageBox MB_OK "Want $3 , found $4" 
  IntCmp $4 $3 FoundNew FoundOld FoundNew
 
NoFound:
  #MessageBox MB_OK "JRE not found"
  Push "0"
  Goto DetectJREEnd
 
FoundOld:
  #MessageBox MB_OK "JRE too old: $3 is older than $4"
;  Push ${TEMP2}
  Push "-1"
  Goto DetectJREEnd  
FoundNew:
  #MessageBox MB_OK "JRE is new: $3 is newer than $4"
 
  Push "$2\bin\java.exe"
;  Push "OK"
;  Return
   Goto DetectJREEnd
DetectJREEnd:
    ; Top of stack is return value, then r4,r3,r2,r1
    Exch    ; => r4,rv,r3,r2,r1,r0
    Pop $4  ; => rv,r3,r2,r1r,r0
    Exch    ; => r3,rv,r2,r1,r0
    Pop $3  ; => rv,r2,r1,r0
    Exch    ; => r2,rv,r1,r0
    Pop $2  ; => rv,r1,r0
    Exch    ; => r1,rv,r0
    Pop $1  ; => rv,r0
    Exch    ; => r0,rv
    Pop $0  ; => rv 
FunctionEnd
