#  Draco Econometrics
#  Copyright (C) 2009 Approximatrix, LLC
#  http://www.approximatrix.com
#  
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

include Java

import com.approximatrix.draco.application.plugin.PluginType
import com.approximatrix.draco.datamodel.StatTableModel

require 'pengine/model/plugin'

# Here you need to define your plugin
#register(Plugin::PluginSkeleton) # For example

module Plugin
  
  class IOPluginSkeleton < PluginSkeleton
  	
  	include com.approximatrix.draco.application.plugin.IRubyIOPlugin

    attr_reader :failure
    
    def initialize(model)
      #disable_options_init
      super(model,false)
      #enable_options_init
      
      @file = nil
      @filename = nil
      @import = false
      @streams = false
      
      @data = nil
      @variable_names = nil
      @failure = nil
      
    end
    
    def set_import(value=true)
      @import = value
      value
    end
    
    def set_export(value=true)
      set_import(false)
      value
    end
    
    def presave
      
    end
    
    def save
    
    end

    def preload
      
    end   
        
    def load
    
    end
    
    def get_filter
      "Text Files;txt"
    end
    
    def set_filename(filename)
    	@filename = filename
    end
    
    def get_filename()
    	@filename
    end
    
    def run()
    	open() unless @file
    	
    	#begin
    	  load if @import
          save unless @import
    	
    	  set_model if @import
    	#ensure
    	  close
    	#end
    end
    
    def preliminary()
      open()
      #begin
        preload if @import
        presave unless @import
      #rescue e
        close()
      #  throw e
      #end
      
      initialize_options
      
    end
    
    def get_model()
      @model      
    end
    
    def IOPluginSkeleton.type
      PluginType.io
    end
    
    private
    
    def open()
      if(@streams)
        @file = java.io.FileInputStream.new(java.io.File.new(@filename)) if @import
        @file = java.io.FileOutputStream.new(java.io.File.new(@filename)) unless @import 
      else
        @file = File.new(@filename,"r") if @import
        @file = File.new(@filename,"w") unless @import
      end
    end
    
    def close
      @file.close
      @file = nil
    end
    
    def set_model
    	if(@data)
    	
    	    # If there are no assigned variable names, do this now.
    		if(not @variable_names)
    			@variable_names = []
    			@data.columns.size.times do |i|
    				@variable_names << "import"+i.to_s
    			end
    		end 
    	
    		var_jarray = @variable_names.to_java :String
    		@model = StatTableModel.new(@data.to_java,var_jarray)
    		return @model
    	end
    	return nil
    end
    
    def disable_options_init
      instance_eval(" 
            alias_method :hold_initialize_options, :initialize_options
            def initialize_options
               PluginOptions.new
             end")       
    end
    
    def enable_options_init
      IOPluginSkeleton.class_eval("alias_method :initialize_options, :hold_initialize_options")
    end
    
  end
  
end # module Plugin