#  Draco Econometrics
#  Copyright (C) 2009 Approximatrix, LLC
#  http://www.approximatrix.com
#  
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

require 'pengine/model/io_plugin'

import org.apache.poi.hssf.usermodel.HSSFWorkbook
import org.apache.poi.xssf.usermodel.XSSFWorkbook

import org.apache.poi.ss.usermodel.Cell

class ExcelIO < Plugin::IOPluginSkeleton
  @@NAME = "Microsoft Excel (XLS)"
  
  def initialize(model)
    super(model)
    @filename = nil
    @worksheets = []
    
    @streams = true
  end
  
  def get_filter
    "Excel Files;xls"
  end
  
  def save
    workbook = new_workbook
    sheet = workbook.createSheet("Data")
    
    start_row = 0
    
    if(@options.get_option("Variable Names in First Row"))
      header_row = sheet.createRow(0)
      @model.variables.length.times do |i|
        cell = header_row.createCell(i)
        cell.setCellValue(@model.variables[i])
        start_row = 1
      end
    end
      
    @model.variables.length.times do |i|
      variable = @model.get_variable(@model.variables[i])
      
      irow = start_row
      variable.each do |value|
        row = nil
        row = sheet.getRow(irow)
        row = sheet.createRow(irow) if row.nil?
        
        cell = row.createCell(i)
        
        if(value.is_a? Numeric)
          cell.setCellValue(value)
        end  
      
        irow = irow + 1
      end
    end

	# Finally, write out our spreadsheet
	workbook.write(@file) 
    
  end
  
  def load
    workbook = new_workbook(@file)
    sheet = workbook.getSheet(@options.get_option("Worksheet to Import:"))
    return nil if sheet.nil?
    
    
    startdata = 0
    stopdata = sheet.getLastRowNum()
    
    # Check if a stop row is specified
    if(@options.get_option('Last Row Index to Import (-1 for all):') > 0)
      stopdata = [stopdata, @options.get_option('Last Row Index to Import (-1 for all):')-1].min
    end
    
    last_column = @options.get_option('Last Column Index to Import (-1 for all):')
    
    columns = [0,0]
    headers = nil
    
    # First we need to determine the number of rows and columns to import
    (startdata..stopdata).each do |ir|
      row = sheet.getRow(ir)
      
      if(@options.get_option("Variable Names in First Row"))
      
      	columns = [row.getFirstCellNum(), row.getLastCellNum()]
      	columns[1] = [columns[1],last_column].min if(last_column > 0)
      	
      	startdata = 1
      	
      	headers = []
      	(columns[0]..columns[1]-1).each do |ix|
      	  begin 
      	    headers << row.getCell(ix).getStringCellValue()
      	  rescue
      	    headers << 'excel'+ix.to_s
      	  end 
      	end
      	
      	break
      end
      
      columns[0] = [columns[0],row.getFirstCellNum()].min
      columns[1] = [columns[1],row.getLastCellNum()].max
      columns[1] = [columns[1],last_column].min if(last_column>0)
      
    end
    
    # At this point, if no headers are available, create them
    if(headers.nil?)
      headers = []
      (columns[0]..columns[1]-1).each do |ix|
        headers << 'excel'+ix.to_s
      end
    end
    
    # Now perform the data import
    data = []
    (startdata..stopdata).each do |ir|
      row = sheet.getRow(ir)
      
      drow = []
      
      (columns[0]..columns[1]-1).each do |ix|
        begin
          if(row.getCell(ix).getCellType() == Cell::CELL_TYPE_BLANK)
            drow << nil
          else
            drow << row.getCell(ix).getNumericCellValue()
          end
        rescue
          drow << nil
        end
      end
      data << drow
      
    end
    
    # Convert the drow array into a matrix for the data model
    @data = Matrix.rows(data)
    
    # And store our new headers
    @variable_names = headers
    
  end
  
  def initialize_options
    if(@import)
      @options.add "Worksheet to Import:", "General Selection", @worksheets[0], list_to_string(@worksheets)
    end
    @options.add "Variable Names in First Row", "Boolean", "false"
    if(@import)
      @options.add "Last Row Index to Import (-1 for all):","Integer","-1"
      @options.add "Last Column Index to Import (-1 for all):","Integer","-1"
    end
  end
  
  def preload
    workbook = new_workbook(@file)
    workbook.getNumberOfSheets().times do |i|
      @worksheets << workbook.getSheetName(i)
    end
  end
  
end

class HSSFExcelIO < ExcelIO
  
  @@NAME = "Microsoft Excel (XLS)"
  
  def new_workbook(file=nil)
    wb = HSSFWorkbook.new if file.nil?
    wb = HSSFWorkbook.new(file) unless file.nil?
    return wb
  end
  
  def get_filter
    "Excel Files;xls"
  end
  
end

register(HSSFExcelIO)

class XSSFExcelIO < ExcelIO
  
  @@NAME = "Microsoft Excel 2007 (XLSX)"
  
  def new_workbook(file=nil)
    wb = nil
    if(file.nil?)
      wb = XSSFWorkbook.new if file.nil?
    else 
      file.close
      wb = XSSFWorkbook.new(@filename) unless @filename.nil?
    end
    return wb
  end
  
  def get_filter
    "Excel 2007 Files;xlsx"
  end
  
end

register(XSSFExcelIO)
