require 'pengine/model/io_plugin'

import com.jmatio.io.MatFileReader
import com.jmatio.io.MatFileWriter
import com.jmatio.types.MLNumericArray
import com.jmatio.types.MLDouble

class MatlabIO < Plugin::IOPluginSkeleton
  @@NAME = "MATLAB (MAT)"
  
  @@ALL1DVAR = "All 1D Variables"
  
  @@EXPORTING = ["Separate Variables","Single Matrix"]
  
  @@BLANKVALUES = ["Set to Zero","Set to -1","Set to NaN","Attempt to Remove"]
  
  @@SINGLE_MATRIX_NAME = "data"
  
  def initialize(model)
    super(model)
    
    @contents = nil
    @toload = []
    
    @oned = []
    
    @streams = false
  end
  
  def get_filter
    "MATLAB Binary Data;mat"
  end
  
  def save
    mldata = []
    varnames = @model.variables
    
    if(@options.get_option("Missing Observations") == @@BLANKVALUES[3])
      mldata = save_non_null
    else
      mldata = save_replace_null
    end
    
    MatFileWriter.new(@filename,mldata)
    
  end
    
  def save_replace_null
   	mldata = []
    varnames = @model.variables
    
    nullvalue = 0.0
    if(@options.get_option("Missing Observations") == @@BLANKVALUES[1])
      nullvalue = -1
    elsif(@options.get_option("Missing Observations") == @@BLANKVALUES[2])
      nullvalue = java.lang.Double::NaN
    end
    
    # Separate variables
  	if(@options.get_option("Export as") == @@EXPORTING[0])
  	
  	  varnames.each do |var|
  	    data = [] 
  	  	@model.get_variable(var).each do |value|
  	  	  if(value.nil?)
  	  	    data << nullvalue
  	  	  else
  	  	    data << value
  	  	  end
  	  	end
  	  	data = data.to_java :Double
  	  	matrix = MLDouble.new(var,data,@model.row_count)
  	  	mldata << matrix
  	  end
  	  
  	# One matrix
  	else
  	  
  	  data = []
  	  
  	  varnames.each do |var|
  	  	@model.get_variable(var).each do |value|
  	  	  if(value.nil?)
  	  	    data << nullvalue
  	  	  else
  	  	    data << value
  	  	  end
  	  	end
  	  end
  	  
  	  data = data.to_java :Double
  	  
  	  matrix = MLDouble.new(@@SINGLE_MATRIX_NAME,data,@model.row_count)
  	  mldata << matrix
    end 
  	mldata
  end
  
  def save_non_null
    mldata = []
    varnames = @model.variables
    
    # Separate variables
  	if(@options.get_option("Export as") == @@EXPORTING[0])
  	  
  	  varnames.each do |var|
  	    data = [] 
  	  	@model.get_variable(var).each do |value|
  	  	  data << value unless value.nil?
  	  	end
  	  	data = data.to_java :Double
  	  	matrix = MLDouble.new(var,data,@model.row_count)
  	  	mldata << matrix
  	  end
  	  
  	else
  
      rmat = @model.to_matrix
      data = []
      rmat.columns.each do |x|
        x.each do |value|
          data << value
        end
      end
      data = data.to_java :Double
  	  matrix = MLDouble.new(@@SINGLE_MATRIX_NAME,data,rmat.rows.length)
  	  mldata << matrix
  	
  	end
    mldata
  end
  
  def load
    if(@options.get_option("Variable to Import") == @@ALL1DVAR)
    
      @variable_names = []
      
      columns = []
      
      @oned.each do |varname|
        var = @contents.get(varname)
        
        if(var.is_a? MLNumericArray)
          @variable_names << varname
          
          data = []
          (var.getSize).times do |i|
            data << var.getReal(i)
          end
          
          columns << data
          
        end
        
      end 
    
      @data = Matrix.columns(columns)
    
    else
     
      data = @contents.get(@options.get_option("Variable to Import"))
      return if data.nil?
      rows = []
      
      if(data.is_a? MLNumericArray)
      	data.getM.times do |i|
      	  row = []
      	  data.getN.times do |j|
      	    row << data.get(i,j)
      	  end
      	  rows << row
      	end
      	
      	@data = Matrix.rows(rows)
      	
      	# Header names too
      	@variable_names = []
      	data.getN.times do |j|
      	  @variable_names << @options.get_option("Variable to Import") + j.to_s
      	end
      	
      end
      
      
    end
    
  end
  
  def initialize_options
    if(@import)
      import_vars = @toload.sort
      import_vars.insert(0,@@ALL1DVAR) unless @oned.empty?
      @options.add "Variable to Import", "General Selection", import_vars[0], list_to_string(import_vars)
    else
      @options.add "Export as", "General Selection", @@EXPORTING[0], list_to_string(@@EXPORTING)
      @options.add "Missing Observations", "General Selection", @@BLANKVALUES[0], list_to_string(@@BLANKVALUES)
    end
  end
  
  def preload
    @contents = MatFileReader.new(@filename).getContent
    # @toload << @@ALL1DVAR
    
    @contents.keySet().each do |key|
      var = @contents.get(key)
      
      if(var.is_a? MLNumericArray)
      
      	if(var.getM != 1 and var.getN != 1)
          @toload << key
      	else
          @oned << key
        end
      
      end
      
    end
     
  end
  
  
end

register(MatlabIO)
