package com.approximatrix.draco.application.plugin;

import java.io.File;
import javax.swing.filechooser.FileFilter;
import java.util.ArrayList;

import com.approximatrix.draco.datamodel.StatTableModel;
import com.approximatrix.draco.io.filters.DracoFileFilter;

public class IOPlugin extends Plugin {
	
	private boolean importing = false;
	
	/**
	 * @param name
	 * @param type
	 * @param script
	 * @param scriptClass
	 */
	public IOPlugin(String name, String scriptClass, String script) {
		super(name, PluginType.io, script, scriptClass);
	}
	
	public IOPlugin(String name, String scriptClass, String script, boolean importing) {
		this(name, scriptClass, script);
		this.importing = importing;
	}
	
	@Override
	public Plugin createInstance() {
		return new IOPlugin(this.getName(),this.getScriptRequire(),this.getScriptClass(), this.isImport());
	}
	
	/** Sets the filename for the object to work with
	 * 
	 * @param fname the filename, with full path
	 */
	public void setFilename(String fname) {
		this.getIORubyObject().set_filename(fname);
	}
	
	/** Returns the filename to be worked with
	 * 
	 * @return a valid filename, or null if none was specified
	 */
	public String getFilename() {
		return this.getIORubyObject().get_filename();
	}
	
	/** Passes to the ruby code that this will be an import
	 * 
	 */
	public void setImport() {
		this.getIORubyObject().set_import();
		importing = true;
	}
	
	/** Returns whether the plugin is importing
	 * 
	 * @return true if it has been designated as "import", false otherwise
	 */
	public boolean isImport() {
		return importing;
	}
	
	/** Passes to the ruby code that this will be an export
	 * 
	 */
	public void setExport() {
		this.getIORubyObject().set_export();
		importing = false;
	}
	
	/** Returns a valid file filter object to aid in selecting a file via
	 * java dialog
	 * 
	 * @return a valid FileFilter object
	 */
	public FileFilter getFileFilter() {
		return new PluginFileFilter(this.getIORubyObject().get_filter());
	}
	
	/** Updates the Ruby object's import/export status based on this
	 * objects private importing value
	 */
	private void updateRubyImportExport() {
		if(importing)
			this.setImport();
		else
			this.setExport();
	}
	
	/** Automatically casts the ruby object properly and returns the IO ruby object
	 * 
	 * @return the plugin object
	 */
	protected IRubyIOPlugin getIORubyObject() {
		try {
			return (IRubyIOPlugin)super.getRubyObject();
		} catch(ClassCastException cce) {
			System.err.println("Unexpected object returned for IO plugin.");
			return null;
		}
	}
	
	/** Executes either the preload or presave code, which might set some
	 * options necessary for the plugin
	 */
	public void executePreliminaryCode() {
		this.getIORubyObject().preliminary();
	}
	
	private class PluginFileFilter extends DracoFileFilter {

		private ArrayList<String> extension = null;
		private String description = null;
		
		public PluginFileFilter(String filter) {
			extension = new ArrayList<String>();
			
			String[] values = filter.split(";");
			description = values[0];
			for(int i=1;i<values.length;i++)
				extension.add(values[i]);
		}
		
		@Override
		public boolean accept(File arg0) {
			
			for(String ext: extension) {
				if(acceptBasedOnExtension(arg0,ext))
					return true;
			}
			return false;
		}

		@Override
		public String getDescription() {
			return description;
		}
		
	}
	
	/** Returns the model as loaded from an Import plugin
	 * 
	 * @return a valid model or null if none was successfully created
	 */
	public StatTableModel getModel() {
		return this.getIORubyObject().get_model();
	}

	@Override
	public void initialize(StatTableModel model) {
		super.initialize(model);
		this.updateRubyImportExport();
	}
}
