/*
 *  Draco Econometrics
 *  Copyright (C) 2009 Approximatrix, LLC
 *  http://www.approximatrix.com
 *  
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.approximatrix.draco.application.plugin.ui.menu;

import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JMenuItem;

import com.approximatrix.draco.application.ApplicationManager;
import com.approximatrix.draco.application.plugin.Plugin;
import com.approximatrix.draco.application.plugin.PluginType;

public abstract class AbstractPluginMenuHandler implements ActionListener {

	public List<JMenuItem> getRegressionPluginMenuItems() {
		return getMenuItemsOfType(PluginType.regression);
	}
	
	public List<JMenuItem> getAnalysisPluginMenuItems() {
		return getMenuItemsOfType(PluginType.analysis);
	}
	
	public List<JMenuItem> getIOPluginMenuItems() {
		return getMenuItemsOfType(PluginType.io);
	}
	
	private List<JMenuItem> getMenuItemsOfType(PluginType type) {
		List<JMenuItem> ret = new ArrayList<JMenuItem>();
		for(Plugin plugin : getListOfType(type)) {
			JMenuItem one = new PluginMenuItem(plugin);
			one.addActionListener(this);
			ret.add(one);
		}
		return ret;
	}
	
	private List<Plugin> getListOfType(PluginType type) {
		List<Plugin> ret = new ArrayList<Plugin>();
		try {
			for(Plugin pi : ApplicationManager.GetPluginRegistry()) {
				if(pi.getType().equals(type))
					ret.add(pi);
			}
		} catch(NullPointerException npe) {
			System.err.println("Plugin registry was null...");
		}
		return ret;
	}
	
}
