/*
 *  Draco Econometrics
 *  Copyright (C) 2006-2009 Approximatrix, LLC
 *  http://www.approximatrix.com
 *  
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.approximatrix.draco.editor.events;

import java.util.EventObject;

import com.approximatrix.draco.editor.ui.RubyEditorPanel;

/** Event used for communicating changes in a file status, such
 * as when a file is saved.
 * @author armstrongj
 */
public class EditorFileStatusEvent extends EventObject {

	public static final int FILE_SAVED = 0;
	public static final int FILE_CLOSED = 1;
	public static final int FILE_MODIFIED = 2;
	public static final int FILE_IN_WORKSHEET_CHANGE = 3;
	public static final int FILE_NAME_CHANGE = 4;
	
	private RubyEditorPanel panel = null;
	private int event = FILE_SAVED;
	
	/** Constructor that sets the source equal to the panel and specifies
	 * the type of event.
	 * @param panel
	 * @param event
	 */
	public EditorFileStatusEvent(RubyEditorPanel panel, int event) {
		super(panel);
		this.panel = panel;
		this.event = event;
	}
	
	public RubyEditorPanel getPanel() {
		return panel;
	}
	
	public int getEvent() {
		return event;
	}
	
}
