/*
 *  Draco Econometrics
 *  Copyright (C) 2006-2009 Approximatrix, LLC
 *  http://www.approximatrix.com
 *  
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.approximatrix.draco.editor.wizard.ui;

import javax.swing.JOptionPane;

import org.pietschy.wizard.Wizard;
import org.pietschy.wizard.WizardModel;
import org.pietschy.wizard.models.BranchingPath;
import org.pietschy.wizard.models.Condition;
import org.pietschy.wizard.models.MultiPathModel;
import org.pietschy.wizard.models.Path;
import org.pietschy.wizard.models.SimplePath;
import org.pietschy.wizard.models.StaticModel;

import com.approximatrix.draco.application.plugin.PluginType;
import com.approximatrix.draco.editor.ui.RubyEditorWindow;
import com.approximatrix.draco.editor.wizard.PluginWizardData;
import com.approximatrix.draco.main.ui.MainWindow;

public class PluginWizardController {

	private MainWindow mw = null;
	private PluginWizardData pwd = null;
	
	public PluginWizardController(MainWindow mw) {
		this.mw = mw;
		this.pwd = new PluginWizardData();
	}
	
	public void runWizard() {
		
		BranchingPath firstPath = new BranchingPath();
		SimplePath optionalPath = new SimplePath();
		SimplePath extPath = new SimplePath();
		SimplePath lastPath = new SimplePath();
		
		firstPath.addStep(new PluginIntroStep(pwd));
		firstPath.addStep(new PluginTypeStep(pwd));
		
		optionalPath.addStep(new VariableSelectionStep(pwd));
		extPath.addStep(new PluginFileFilterStep(pwd));
		
		lastPath.addStep(new PluginOptionsStep(pwd));
		lastPath.addStep(new PluginFinalizeStep(pwd, this.mw));
		
		firstPath.addBranch(optionalPath, new Condition() {
			public boolean evaluate(WizardModel model) {
				if(model instanceof PluginWizardModel) {
					PluginWizardModel pwmodel = (PluginWizardModel)model;
					return pwmodel.getPluginWizardData().getPluginType() != PluginType.io; 
				}
				return true;
			}
		});
		optionalPath.setNextPath(lastPath);

		firstPath.addBranch(extPath, new Condition() {
			public boolean evaluate(WizardModel model) {
				if(model instanceof PluginWizardModel) {
					PluginWizardModel pwmodel = (PluginWizardModel)model;
					return pwmodel.getPluginWizardData().getPluginType() == PluginType.io; 
				}
				return true;
			}
		});
		extPath.setNextPath(lastPath);
		
		PluginWizardModel model = new PluginWizardModel(firstPath, pwd);
		
		Wizard wizard = new Wizard(model);
		//wizard.showInFrame("Draco Plugin Wizard");
		wizard.showInDialog("Draco Plugin Wizard", this.mw, true);

		if(pwd.getPluginText() != null) {
			RubyEditorWindow.OpenEditorTab(pwd.getPluginText(), true, mw);
		} else {
			JOptionPane.showMessageDialog(mw, "The plugin failed to intialize...","Plugin System Error",JOptionPane.ERROR_MESSAGE);
		}
		
	}
	
	private class PluginWizardModel extends MultiPathModel {

		private PluginWizardData pwd = null;
		
		public PluginWizardModel(Path arg0, PluginWizardData pwd) {
			super(arg0);
			this.pwd = pwd;
		}
		
		public PluginWizardData getPluginWizardData() {
			return this.pwd;
		}
		
	}
	
}
