package com.approximatrix.draco.fit;

import java.util.ArrayList;
import java.util.List;

import com.approximatrix.draco.datamodel.StatTableModel;

public abstract class AbstractFit implements LSFit {

	/** The message if a failure has occurred */
	protected String failure_message = null;
	
	/** The results of the fit */
	protected String results = null;
	
	/** Flag indicated success (let's be optimistic) */
	protected boolean success = true;
	
	/** The list of intermediate messages */
	private ArrayList<String> messages = null;
	
	/** The base model */
	private StatTableModel model = null;
	
	protected AbstractFit(StatTableModel mdl) {
		setModel(mdl);
	}
	
	/** Sets the base model
	 *  
	 * @param mdl a valid model
	 */
	protected void setModel(StatTableModel mdl) {
		model = mdl;
	}
	
	/** Adds a message to the message list
	 * 
	 * @param message the message to add to the list
	 */
	protected void addMessage(String message) {
		if(messages == null)
			messages = new ArrayList<String>();
		messages.add(message);
	}

	public List<String> getMessages() {
		return messages;
	}

	public StatTableModel getModel() {
		return model;
	}

	public boolean isSuccessful() {
		return success;
	}
	
	
}
