/*
 *  Draco Econometrics
 *  Copyright (C) 2006-2007 Approximatrix, LLC
 *  http://www.approximatrix.com
 *  
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.approximatrix.draco.fit;

import java.util.List;

import com.approximatrix.draco.fit.statistics.ExtendedStatistic;

/** Iterface for classes that perform regressions using an interative procedure
 * 
 * @author jeff
 *
 */
public interface LSIterative extends LSFit {

	/** Returns the current iteration number
	 * 
	 * @return the iterative step
	 */
	public int getCurrentIteration();
	
	/** Returns a measure of change for a single iteration that can
	 * be used for tolerance limiting
	 * 
	 * @return a measure of the amount of change ocmputed in this step
	 */
	public double getIterationChange();
	
	/** Returns a new LSIterative object representing the next iteration
	 * 
	 * @return the next iteration
	 */
	public LSIterative getNextIteration();
	
	/** Returns the list of extended statistics without performing computations */
	public List<ExtendedStatistic> getExtendedStatisticsWithoutCompute(); 
}
