/*
 *  Draco Econometrics
 *  Copyright (C) 2008-2009 Approximatrix, LLC
 *  http://www.approximatrix.com
 *  
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.approximatrix.draco.fit.functions;

import com.approximatrix.draco.math.functions.ErrorFunction;

public class ProbitLikelihood extends SimpleLikelihood implements LikelihoodFunction {

	private static final String FUNCTION_NAME = "Probit";
	
	public ProbitLikelihood() {
		super();
	}
	
	public ProbitLikelihood(double[][] independent, double[] dependent) {
		super(independent, dependent);
	}
	
	/** Returns the CDF value at the given variable position based on the specified
	 * parameters.
	 * 
	 * @see http://en.wikipedia.org/wiki/Normal_distribution#Cumulative_distribution_function
	 * @see http://en.wikipedia.org/wiki/Probit
	 */
	public double evaluateFunction(double[] parameters, double[] variables) {
		//double cdf = 0.5*(1.0+Num.erf(calculateLinearTerm(parameters,variables)/Math.sqrt(2.0)));
		//if(cdf != 0.0)
			return 1.0;
		//else
		//	return 0.0;
	}

	public double evaluateLikelihoodRow(double[] parameters,
			double[] independent, double dependent) {

		double lt = calculateLinearTerm(parameters,independent);
		double cdf = 0.5*(1.0+ErrorFunction.Erf(lt/Math.sqrt(2.0)));
		
		return dependent*Math.log(cdf)+(1.0-dependent)*Math.log(1.0-cdf);
	}

	public String getLikelihoodType() {
		return FUNCTION_NAME;
	}

	public double evaluatePDF(double[] parameters, double[] variables) {
		double linterm = this.calculateLinearTerm(parameters, variables);
		double exponential = Math.exp(linterm*linterm/(-2.0));
		return exponential/Math.sqrt(2.0*Math.PI);
	}
	
	public double evaluateCDF(double[] parameters, double[] variables) {
		double linterm = this.calculateLinearTerm(parameters, variables);
		return 0.5*(1.0+ErrorFunction.Erf(calculateLinearTerm(parameters,variables)/Math.sqrt(2.0)));
	}

}
