/*
 *  Draco Econometrics
 *  Copyright (C) 2009 Approximatrix, LLC
 *  http://www.approximatrix.com
 *  
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.approximatrix.draco.general.ui;

import java.awt.Container;
import java.awt.Dimension;
import java.awt.FontMetrics;
import java.text.BreakIterator;

import javax.swing.JLabel;
import javax.swing.SwingUtilities;

public class WrappingLabel extends JLabel {
	private String original = null; 

	public WrappingLabel(String text) {
		super(text);
	}

	@Override
	public void doLayout() {
		if(original == null) original = super.getText();
		wrapLabelText();
		super.doLayout();
	}

	@Override
	public void setSize(Dimension d) {
		super.setSize(d);
		if(original == null) original = super.getText();
		wrapLabelText();
	}

	@Override
	public void setSize(int width, int height) {
		super.setSize(width, height);
		if(original == null) original = super.getText();
		wrapLabelText();
	}

	private void wrapLabelText() {

		FontMetrics fm = this.getFontMetrics(this.getFont());
		Container container = this.getParent();
		if(container == null) return;

		int containerWidth = container.getWidth();

		BreakIterator boundary = BreakIterator.getWordInstance();
		boundary.setText(original);

		StringBuffer trial = new StringBuffer();
		StringBuffer real = new StringBuffer("<html>");

		int start = boundary.first();
		for (int end = boundary.next(); end != BreakIterator.DONE;
		start = end, end = boundary.next()) {
			String word = original.substring(start,end);
			trial.append(word);
			int trialWidth = SwingUtilities.computeStringWidth(fm,
					trial.toString());
			if (trialWidth > containerWidth) {
				trial = new StringBuffer(word);
				real.append("<br>");
			}
			real.append(word);
		}

		real.append("</html>");

		super.setText(real.toString());
	}

}
