/*
 *  Draco Econometrics
 *  Copyright (C) 2006-2007 Approximatrix, LLC
 *  http://www.approximatrix.com
 *  
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.approximatrix.draco.io;

import javax.swing.JFrame;


import com.approximatrix.draco.main.ui.MainWindow;
import com.approximatrix.draco.io.XMLio.LoadWorksheet;
import com.approximatrix.draco.io.ui.IOFileWorkerDialog;
import com.approximatrix.draco.io.ui.IOWorker;

public class PreLoadODS implements IOWorker {

	private static final String PRELOAD_TITLE = "Opening ODS File...";
	
	private static final String[] LOAD_STEPS = {"Accessing OpenDocument File",
												"Loading into Memory",
												"Determining Available Worksheets"};
	private static final int TOTAL_STEPS = 3;
	
	private String filename = null;
	private int current_step = 0;
	private MainWindow parent = null;
	
	private LoadWorksheet lw = null;
	private boolean success = true;
	
	private String[] available_sheets = null;
	
	public PreLoadODS(String filename, MainWindow parent) {
		this.filename = filename;
		current_step = 1;
		this.parent = parent;
	}
	
	public void load() {
		new IOFileWorkerDialog(this).setVisible(true);
	}
	
	public String getWorkerTitle() {
		return PRELOAD_TITLE;
	}

	public void doStep() {
		switch(current_step) {
			case 1:
				lw = new LoadWorksheet(filename,parent);
				success = (lw != null);
				break;
			case 2:
				success = lw.openIntoMemory();
				break;
			case 3:
				available_sheets = lw.getTables();
				success = (available_sheets != null);
				break;
			default:
				success = false;
		}
		current_step++;
	}

	public int getStepNumber() {
		return current_step;
	}

	public int getTotalSteps() {
		return TOTAL_STEPS;
	}

	public String getStatus() {
		return LOAD_STEPS[getStepNumber()-1];
	}

	public JFrame getParentWindow() {
		return parent;
	}

	public boolean stepSuccess() {
		return success;
	}

	public String[] getAvailableSheets() {
		return available_sheets;
	}
	
	public String getError() {
		if(lw != null) 
			return lw.getError();
		else
			return null;
	}
}
