/*
 *  Draco Econometrics
 *  Copyright (C) 2006-2007 Approximatrix, LLC
 *  http://www.approximatrix.com
 *  
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.approximatrix.draco.io;

import javax.swing.JFrame;


import com.approximatrix.draco.analysis.ui.AnalysisChildDialog;
import com.approximatrix.draco.application.ApplicationManager;
import com.approximatrix.draco.application.plugin.ui.PluginDataProvidingWindow;
import com.approximatrix.draco.main.ui.MainWindow;
import com.approximatrix.draco.main.ui.WorksheetHandler;
import com.approximatrix.draco.plot.ui.PlotDialog;
import com.approximatrix.draco.editor.ui.RubyEditorWindow;
import com.approximatrix.draco.fit.ui.FitChildDialog;
import com.approximatrix.draco.general.ui.InfoWindow;
import com.approximatrix.draco.io.sql.DracoSQLWrite;
import com.approximatrix.draco.io.ui.IOFileWorkerDialog;
import com.approximatrix.draco.io.ui.IOWorker;

public class SaveDracoSQL implements IOWorker, DracoSaver {

	private static final String SAVE_WORKSHEET_TITLE = "Save "+ApplicationManager.APP_NAME+" File...";
	private static final String[] SAVE_STEPS = {"Initializing SQL Connection",
												"Saving Worksheet Data",
												"Storing Constants",
												"Saving Analyses",
												"Saving Regressions",
												"Saving Plots and Charts",
												"Saving Texts",
												"Writing Plugin Information",
												"Finalizing Document"};
	private static final int TOTAL_STEPS = 9;
	
	private int current_step = 0;
	private String filename = null;
	private WorksheetHandler parent = null;
	
	private DracoSQLWrite sql = null; 
	private boolean success = true;
	
	private String error = null;
	
	public SaveDracoSQL(String filename, WorksheetHandler parent) {
		this.filename = filename;
		current_step = 1;
		this.parent = parent;
	}
	
	public void save() {
		new IOFileWorkerDialog(this).setVisible(true);
	}
	
	public String getWorkerTitle() {
		return SAVE_WORKSHEET_TITLE;
	}

	public void doStep() {
		try {
			switch(current_step) {
				case 1:
					sql = new DracoSQLWrite(filename);
					break;
				case 2:
					success = sql.saveWorksheet(parent.getWindowTableModel());	
					break;
				case 3:
					success = sql.saveConstants(parent.getWindowTableModel());
					break;
				case 4:
					success = sql.saveAnalyses(parent.getWindowsByClass(AnalysisChildDialog.class));
					break;
				case 5:
					success = sql.saveRegressions(parent.getWindowsByClass(FitChildDialog.class));
					break;
				case 6:
					success = sql.savePlots(parent.getWindowsByClass(PlotDialog.class));
					break;
				case 7:
					success = sql.saveTexts(parent.getWindows(InfoWindow.class), parent.getWindows(RubyEditorWindow.class));
					break;
				case 8:
					success = sql.savePlugins(parent.getWindowsByClass(PluginDataProvidingWindow.class));
					break;
				case 9:
					sql.close();
					break;
				default:
					success = false;
			}
		} catch (IllegalStateException ise) {
			System.err.println("NestedVM failed during save...");
			ise.printStackTrace();
			success = false;
			error = "Problem during data saving.\n" +
			"Please report the issue to\n"   +
			"support@approximatrix.com";
		}
		current_step++;
	}

	public int getStepNumber() {
		return current_step;
	}

	public int getTotalSteps() {
		return TOTAL_STEPS;
	}

	public String getStatus() {
		return SAVE_STEPS[current_step-1];
	}
	
	public JFrame getParentWindow() {
		if(parent instanceof JFrame)
			return (JFrame)parent;
		else
			return null;
	}
	
	public boolean stepSuccess() {
		return success;
	}
	
	public String getError() {
		if(sql != null) 
			return sql.getError();
		else
			return error;
	}
	
}
