package com.approximatrix.draco.fit;

import org.junit.Test;
import static org.junit.Assert.*;

import com.approximatrix.draco.fit.functions.LogitLikelihood;
import com.approximatrix.draco.fit.functions.ProbitLikelihood;

public class SimpleMLEFitTest extends RegressionTestSkeleton {

	protected static final double DELTA = 1.0E-4;
	protected static final double COV_DELTA = 5.0E-2;
	
	@Test
	public void testRlogit_Probit() {
		String[] indep = new String[3]; 
		indep[0] = "gre"; indep[1] = "gpa"; indep[2] = "topnotch";
		SimpleMLEFit fit = new SimpleMLEFit(rlogit, "admit", indep, ProbitLikelihood.class);
		fit.perform();
		
		double[] coef = fit.getCoef();
		assertCloseTo(-2.7978830,coef[0],DELTA);
		assertCloseTo( 0.0015244,coef[1],DELTA);
		assertCloseTo( 0.4009847,coef[2],DELTA);
		assertCloseTo( 0.2730331,coef[3],DELTA);
		
		double[] var = fit.getCoefVariances();
		assertCloseTo(0.6476352*0.6476352,var[0],COV_DELTA);
		assertCloseTo(0.0006404*0.0006404,var[1],COV_DELTA); // <- A nasty, small number
		assertCloseTo(0.1948032*0.1948032,var[2],COV_DELTA); // <- Its close, but not within 1%
		assertCloseTo(0.1803284*0.1803284,var[3],COV_DELTA); // <- Its close, but not within 1%
		
		assertNotNull(fit.getResults());
	}
	
	@Test
	public void testRlogit_Logit() {
		String[] indep = new String[3]; 
		indep[0] = "gre"; indep[1] = "gpa"; indep[2] = "topnotch";
		SimpleMLEFit fit = new SimpleMLEFit(rlogit, "admit", indep, LogitLikelihood.class);
		fit.perform();
		
		double[] coef = fit.getCoef();
		assertCloseTo(-4.600814,coef[0],DELTA);
		assertCloseTo( 0.002477,coef[1],DELTA*10); // <- This one differs from R, but it isn't "wrong"
		assertCloseTo( 0.667556,coef[2],DELTA);
		assertCloseTo( 0.437224,coef[3],DELTA);
		
		double[] var = fit.getCoefVariances();
		assertCloseTo(1.096379*1.096379,var[0],COV_DELTA);
		assertCloseTo(0.001070*0.001070,var[1],COV_DELTA);
		assertCloseTo(0.325259*0.325259,var[2],COV_DELTA);
		assertCloseTo(0.291853*0.291853,var[3],COV_DELTA);
		
		assertNotNull(fit.getResults());
	}
	
}
