package com.approximatrix.draco.fit.functions;

import static org.junit.Assert.*;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

public class LogitLikelihoodTest extends LikelihoodTestSkeleton {

	private static final double[] Z = {
		0.0,
		0.1,
		0.2,
		0.3,
		0.4,
		0.5,
		0.6,
		0.7,
		0.8,
		0.9,
		1.0,
		1.1,
		1.2,
		1.3,
		1.4,
		1.5,
		1.6,
		1.7,
		1.8,
		1.9,
		2.0,
		2.1,
		2.2,
		2.3,
		2.4,
		2.5,
		2.6,
		2.7,
		2.8,
		2.9,
		3.0
	};
	
	private static final double[] PDF = {
		   0.250000,
		   0.249376,
		   0.247517,
		   0.244458,
		   0.240261,
		   0.235004,
		   0.228784,
		   0.221713,
		   0.213910,
		   0.205500,
		   0.196612,
		   0.187370,
		   0.177894,
		   0.168298,
		   0.158685,
		   0.149146,
		   0.139764,
		   0.130606,
		   0.121729,
		   0.113180,
		   0.104994,
		   0.097195,
		   0.089800,
		   0.082820,
		   0.076255,
		   0.070104,
		   0.064358,
		   0.059008,
		   0.054038,
		   0.049434,
		   0.045177
	};
	
	private static final double[] CDF = {
		   0.50000,
		   0.52498,
		   0.54983,
		   0.57444,
		   0.59869,
		   0.62246,
		   0.64566,
		   0.66819,
		   0.68997,
		   0.71095,
		   0.73106,
		   0.75026,
		   0.76852,
		   0.78583,
		   0.80218,
		   0.81757,
		   0.83202,
		   0.84553,
		   0.85815,
		   0.86989,
		   0.88080,
		   0.89090,
		   0.90025,
		   0.90888,
		   0.91683,
		   0.92414,
		   0.93086,
		   0.93703,
		   0.94268,
		   0.94785,
		   0.95257
	};
	
	private LogitLikelihood ll = null;
	
	@Before
	public void setUp() throws Exception {
		ll = new LogitLikelihood(null,null);
	}

	@After
	public void tearDown() throws Exception {
		ll = null;
	}	
	
	@Test
	public void testGetLikelihoodType() {
		assertEquals("Logit",ll.getLikelihoodType());
	}

	@Test
	public void testEvaluatePDF() {
		double[] variables = {0.0};
		for(int i=0;i<Z.length;i++) {
			double[] parameters = {Z[i],1.0};
			double value = ll.evaluatePDF(parameters, variables);
			assertWithin(PDF[i],value,0.00001);
		}		
	}

	@Test
	public void testEvaluateCDF() {
		double[] variables = {0.0};
		for(int i=0;i<Z.length;i++) {
			double[] parameters = {Z[i],1.0};
			double value = ll.evaluateCDF(parameters, variables);
			assertWithin(CDF[i],value,0.0001);
		}
	}

	@Test
	public void testCalculateLinearTerm() {
		double[] parameters = {1.0,2.0};
		double[] variables = {1.0};
		assertEquals(3.0,ll.calculateLinearTerm(parameters, variables));
		
		double[] parameters2 = {1.0,-2.0};
		assertEquals(-1.0,ll.calculateLinearTerm(parameters2, variables));
		
		double[] parameters3 = new double[100];
		parameters3[0] = 1.0;
		double[] variables3 = new double[99];
		double value = -1.0;
		for(int i=0;i<variables3.length;i++) {
			variables3[i] = 2.0;
			parameters3[i+1] = value;
			value = -1.0*value;
		}
		assertEquals(-1.0,ll.calculateLinearTerm(parameters3, variables3));
	}
	
}
