package com.approximatrix.draco.fit.functions;

import static org.junit.Assert.*;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

public class ProbitLikelihoodTest extends LikelihoodTestSkeleton {

	private static final double[] Z = {
		0.0,
		0.1,
		0.2,
		0.3,
		0.4,
		0.5,
		0.6,
		0.7,
		0.8,
		0.9,
		1.0,
		1.1,
		1.2,
		1.3,
		1.4,
		1.5,
		1.6,
		1.7,
		1.8,
		1.9,
		2.0,
		2.1,
		2.2,
		2.3,
		2.4,
		2.5,
		2.6,
		2.7,
		2.8,
		2.9,
		3.0
	};
	
	private static final double[] PDF = {
		   0.3989423,
		   0.3969525,
		   0.3910427,
		   0.3813878,
		   0.3682701,
		   0.3520653,
		   0.3332246,
		   0.3122539,
		   0.2896916,
		   0.2660852,
		   0.2419707,
		   0.2178522,
		   0.1941861,
		   0.1713686,
		   0.1497275,
		   0.1295176,
		   0.1109208,
		   0.0940491,
		   0.0789502,
		   0.0656158,
		   0.0539910,
		   0.0439836,
		   0.0354746,
		   0.0283270,
		   0.0223945,
		   0.0175283,
		   0.0135830,
		   0.0104209,
		   0.0079155,
		   0.0059525,
		   0.0044318
	};
	
	private static final double[] CDF = {
		0.5,
		0.5398,
		0.5793,
		0.6179,
		0.6554,
		0.6915,
		0.7257,
		0.758,
		0.7881,
		0.8159,
		0.8413,
		0.8643,
		0.8849,
		0.9032,
		0.9192,
		0.9332,
		0.9452,
		0.9554,
		0.9641,
		0.9713,
		0.9772,
		0.9821,
		0.9861,
		0.9893,
		0.9918,
		0.9938,
		0.9953,
		0.9965,
		0.9974,
		0.9981,
		0.9987
	};
	
	
	private ProbitLikelihood pl = null;
	
	@Before
	public void setUp() throws Exception {
		pl = new ProbitLikelihood(null,null);
	}

	@After
	public void tearDown() throws Exception {
		pl = null;
	}

	@Test
	public void testEvaluatePDF() {
		double[] variables = {0.0};
		for(int i=0;i<Z.length;i++) {
			double[] parameters = {Z[i],1.0};
			double value = pl.evaluatePDF(parameters, variables);
			assertWithin(PDF[i],value,0.00001);
		}		
	}

	@Test
	public void testEvaluateCDF() {
		double[] variables = {0.0};
		for(int i=0;i<Z.length;i++) {
			double[] parameters = {Z[i],1.0};
			double value = pl.evaluateCDF(parameters, variables);
			assertWithin(CDF[i],value,0.0001);
		}
	}
	
	@Test
	public void testGetLikelihoodType() {
		assertEquals("Probit",pl.getLikelihoodType());
	}

	@Test
	public void testCalculateLinearTerm() {
		double[] parameters = {1.0,2.0};
		double[] variables = {1.0};
		assertEquals(3.0,pl.calculateLinearTerm(parameters, variables));
		
		double[] parameters2 = {1.0,-2.0};
		assertEquals(-1.0,pl.calculateLinearTerm(parameters2, variables));
		
		double[] parameters3 = new double[100];
		parameters3[0] = 1.0;
		double[] variables3 = new double[99];
		double value = -1.0;
		for(int i=0;i<variables3.length;i++) {
			variables3[i] = 2.0;
			parameters3[i+1] = value;
			value = -1.0*value;
		}
		assertEquals(-1.0,pl.calculateLinearTerm(parameters3, variables3));
	}
	
}
